import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.io.*;
import java.util.List;
import javax.swing.*;
import javax.swing.SwingUtilities;
import javax.swing.filechooser.*;

public class Game  extends World
{

    public Vector gravity = new Vector(Math.PI / 2, 1.2);
    public Vector up;
    
    public WavPlayer theme = new WavPlayer();
    public boolean intro = true;
    private boolean begin = false;
    
    public boolean debug = false; // CHANGE ME!!!!!
    
    public Score score = new Score();
    private Logo logo = new Logo();
    private boolean logoNotRemoved = true;
    public boolean done = true;
    public Mario player;
    public JFileChooser fileLoader = new JFileChooser();
    public File file;
    public JPanel panelHandler = new JPanel();
    public boolean invincible = false;

    private String version = "0001";
    
    public Game()
    {    
        // Create a new world with 20x20 cells with a cell size of 10x10 pixels.
        super(800, 600, 1); 
        
        Greenfoot.setSpeed(50);
        
        up = new Vector(gravity.direction + Math.PI, gravity.magnitude);
                
        makeCloud(7);
        
        if (debug) addObject(new Helper(this), 16, 16);
        
        addObject(score, 384, 16);
        addObject(logo, (int)(getWidth() / 2), (int)(getHeight() / 2));
        
        setPaintOrder(
            Logo.class,
            Score.class,
            Helper.class,
            Point.class,
            TopCloud.class,
            Hurt.class,
            Dead.class,
            Coin.class,
            Fireball.class,
            Turn.class,
            PowerUp.class,
            Goal.class,
            Brick.class,
            Block.class,
            Floor.class,
            Weakpoint.class,
            TreeSmall.class,
            People.class,
            SuperYoshi.class,
            Cloud.class
        );
        
        fileLoader.addChoosableFileFilter(new GameFileFilter());
        fileLoader.setAcceptAllFileFilterUsed(false);
        
        WavPlayer preIntro = new WavPlayer();
        preIntro.open("Intro.wav");
        preIntro.setLoops(1);
        preIntro.play();
    }
    
    private void makeCloud(int amount)
    {
        for (int i = 0; i < amount; i++)
        {
            addObject(new TopCloud(), (int)(Math.random() * getWidth()), (int)(Math.random() * getHeight()));
            addObject(new Cloud(), (int)(Math.random() * getWidth()), (int)(Math.random() * getHeight()));
        }
    }
    
    public void goalReached(Goal target)
    {
        removeObject(target);
        theme.stop();
        theme.close();
        theme.open("Goal.wav");
        theme.setLoops(1);
        theme.play();
        done = true;
        intro = true;
        Greenfoot.delay(400);
    }
    
    public void act()
    {
        if (logoNotRemoved)
        {
            logoNotRemoved = false;
            removeObject(logo);
        }
        if (!debug)
        {
            if (begin)
            {
                if (!theme.getIsPlaying())
                {
                    if (done)
                    {
                        if (open())
                        {
                            if (load()) done = false;
                        } else Greenfoot.stop();
                    } else {
                        if (intro)
                        {
                            theme.open("Begin.wav");
                            theme.setLoops(1);
                            theme.play();
                            intro = false;
                        } else {
                            theme.close();
                            theme.open("Theme.wav");
                            theme.setLoops(-1);
                            theme.play();
                        }
                    }
                    invincible = false;
                }
            } else {
                begin = true;
                addObject(new Point(this), 16, (debug ? 48 : 16));
            }
        }
    }
    
    public boolean open()
    {
        int returnVal = fileLoader.showOpenDialog(panelHandler);
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            file = fileLoader.getSelectedFile();
            return true;
        } else {
            return false;
       }
    }
    
    private void error(int times)
    {
        theme.close();
        theme.open("Dead.wav");
        theme.setLoops(times);
        theme.play();
    }
    
    //Brick
    //Floor
    //Block
    //Weakpoint
    //Fireball
    //TreeSmall
    //TreeBig
    //Goal
    //Turn
    //PowerUp
    //Coin
    //Plant
    //Goomba
    //Koopa
    //Bowser
    //Player
    //Gravity
    
    public void save()
    {
        if (player == null)
        {
            error(2);
            return;
        } else if (player.getWorld() == null) {
            error(2);
            return;
        }
        try
        {
            FileWriter saver = new FileWriter(file);
            String data = "smg2d!!!" + version + "\n";
            
            List bricks = getObjects(Brick.class);
            Brick brick;
            for (Object _brick : bricks)
            {
                brick = (Brick)_brick;
                data += "brick\n";
                data += brick.getX() + "\n";
                data += brick.getY() + "\n";
                data += brick.velocity.direction + "\n";
                data += brick.velocity.magnitude + "\n";
                data += brick.animationStep + "\n";
            }
            
            List floors = getObjects(Floor.class);
            Floor floor;
            for (Object _floor : floors)
            {
                floor = (Floor)_floor;
                data += "floor\n";
                data += floor.getX() + "\n";
                data += floor.getY() + "\n";
                data += floor.radius + "\n";
            }
            
            List blocks = getObjects(Block.class);
            Block block;
            for (Object _block : blocks)
            {
                block = (Block)_block;
                data += "block\n";
                data += block.getX() + "\n";
                data += block.getY() + "\n";
            }
            
            List weakpoints = getObjects(Weakpoint.class);
            Weakpoint weakpoint;
            for (Object _weakpoint : weakpoints)
            {
                weakpoint = (Weakpoint)_weakpoint;
                data += "weakpoint\n";
                data += weakpoint.getX() + "\n";
                data += weakpoint.getY() + "\n";
            }
            
            List fireballs = getObjects(Fireball.class);
            Fireball fireball;
            for (Object _fireball : fireballs)
            {
                fireball = (Fireball)_fireball;
                data += "fireball\n";
                data += fireball.getX() + "\n";
                data += fireball.getY() + "\n";
                data += fireball.velocity.direction + "\n";
                data += fireball.velocity.magnitude + "\n";
            }
            
            List treeSmalls = getObjects(TreeSmall.class);
            TreeSmall treeSmall;
            for (Object _treeSmall : treeSmalls)
            {
                treeSmall = (TreeSmall)_treeSmall;
                data += "treesmall\n";
                data += treeSmall.getX() + "\n";
                data += treeSmall.getY() + "\n";
            }
            
            List treeBigs = getObjects(TreeBig.class);
            TreeBig treeBig;
            for (Object _treeBig : treeBigs)
            {
                treeBig = (TreeBig)_treeBig;
                data += "treebig\n";
                data += treeBig.getX() + "\n";
                data += treeBig.getY() + "\n";
            }
            
            List goals = getObjects(Goal.class);
            Goal goal;
            for (Object _goal : goals)
            {
                goal = (Goal)_goal;
                data += "goal\n";
                data += goal.getX() + "\n";
                data += goal.getY() + "\n";
            }
            
            List turns = getObjects(Turn.class);
            Turn turn;
            for (Object _turn : turns)
            {
                turn = (Turn)_turn;
                data += "turn\n";
                data += turn.getX() + "\n";
                data += turn.getY() + "\n";
                data += turn.direction.direction + "\n";
                data += turn.direction.magnitude + "\n";
            }
            
            List powerups = getObjects(PowerUp.class);
            PowerUp powerup;
            for (Object _powerup : powerups)
            {
                powerup = (PowerUp)_powerup;
                data += "powerup\n";
                data += powerup.getX() + "\n";
                data += powerup.getY() + "\n";
                data += (powerup.type ? "mushroom" : "flower") + "\n";
            }
            
            List coins = getObjects(Coin.class);
            Coin coin;
            for (Object _coin : coins)
            {
                coin = (Coin)_coin;
                data += "coin\n";
                data += coin.getX() + "\n";
                data += coin.getY() + "\n";
                data += coin.velocity.direction + "\n";
                data += coin.velocity.magnitude + "\n";
            }
            
            List plants = getObjects(Plant.class);
            Plant plant;
            for (Object _plant : plants)
            {
                plant = (Plant)_plant;
                data += "plant\n";
                data += plant.getX() + "\n";
                data += plant.getY() + "\n";
            }
            
            List goombas = getObjects(Goomba.class);
            Goomba goomba;
            for (Object _goomba : goombas)
            {
                goomba = (Goomba)_goomba;
                data += "goomba\n";
                data += goomba.getX() + "\n";
                data += goomba.getY() + "\n";
                data += (goomba.direction ? "left" : "right") + "\n";
            }
            
            List koopas = getObjects(Koopa.class);
            Koopa koopa;
            for (Object _koopa : koopas)
            {
                koopa = (Koopa)_koopa;
                data += "koopa\n";
                data += koopa.getX() + "\n";
                data += koopa.getY() + "\n";
                data += (koopa.direction ? "left" : "right") + "\n";
            }
            
            List bowsers = getObjects(Bowser.class);
            Bowser bowser;
            for (Object _bowser : bowsers)
            {
                bowser = (Bowser)_bowser;
                data += "bowser\n";
                data += bowser.getX() + "\n";
                data += bowser.getY() + "\n";
                data += (bowser.direction ? "left" : "right") + "\n";
                data += (bowser.face ? "left" : "right") + "\n";
                data += bowser.maxAttack + "\n";
            }
            
            List players = getObjects(Mario.class);
            Mario playerOne = (Mario)players.get(0);
            data += "player\n";
            data += playerOne.getX() + "\n";
            data += playerOne.getY() + "\n";
            
            data += "gravity\n";
            data += gravity.direction + "\n";
            data += gravity.magnitude + "\n";
            
            saver.write(data);
            saver.close();
            
        } catch (IOException e) {
            error(1);
        }
    }
    
    private void reset()
    {
        Class[] items = 
        {
            Brick.class,
            Floor.class,
            Block.class,
            Weakpoint.class,
            Fireball.class,
            TreeSmall.class,
            Goal.class,
            Turn.class,
            PowerUp.class,
            Coin.class,
            People.class
        };
        for (int i = 0; i < 11; i++)
            removeObjects(getObjects(items[i]));
    }
    
    public boolean load()
    {
        reset();
        try
        {
            BufferedReader loader = new BufferedReader(new FileReader(file));

          String testVersion = "smg2d!!!" + version;

            if (loader.readLine().equals(testVersion));
                String cmd;
                String cmd1;
                String cmd2;
                String cmd3;
                String cmd4;
                String cmd5;
                Bowser cmdBoss;
                boolean cmdType;
                while ((cmd = loader.readLine()) != null)
                {
                    if (cmd.equals("brick"))
                    {
                        cmd1 = loader.readLine();
                        cmd2 = loader.readLine();
                        cmd3 = loader.readLine();
                        cmd4 = loader.readLine();
                        cmd5 = loader.readLine();

                        addObject(
                            new Brick(
                                new Vector(
                                    Double.parseDouble(cmd3),
                                    Double.parseDouble(cmd4)
                                ), Integer.parseInt(cmd5)
                            ), Integer.parseInt(cmd1),
                            Integer.parseInt(cmd2)
                        );

                    } else if (cmd.equals("floor")) {
                        cmd1 = loader.readLine();
                        cmd2 = loader.readLine();
                        cmd3 = loader.readLine();

                        addObject(
                            new Floor(Integer.parseInt(cmd3)),
                            Integer.parseInt(cmd1),
                            Integer.parseInt(cmd2)
                        );

                    } else if (cmd.equals("block")) {
                        cmd1 = loader.readLine();
                        cmd2 = loader.readLine();

                        addObject(
                            new Block(),
                            Integer.parseInt(cmd1),
                            Integer.parseInt(cmd2)
                        );

                    } else if (cmd.equals("weakpoint")) {
                        cmd1 = loader.readLine();
                        cmd2 = loader.readLine();

                        addObject(
                            new Weakpoint(),
                            Integer.parseInt(cmd1),
                            Integer.parseInt(cmd2)
                        );

                    } else if (cmd.equals("fireball")) {
                        cmd1 = loader.readLine();
                        cmd2 = loader.readLine();
                        cmd3 = loader.readLine();
                        cmd4 = loader.readLine();

                        addObject(
                            new Fireball(
                                new Vector(
                                    Double.parseDouble(cmd3),
                                    Double.parseDouble(cmd4)
                            )), Integer.parseInt(cmd1),
                            Integer.parseInt(cmd2)
                        );

                    } else if (cmd.equals("goal")) {
                        cmd1 = loader.readLine();
                        cmd2 = loader.readLine();

                        addObject(
                            new Goal(),
                            Integer.parseInt(cmd1),
                            Integer.parseInt(cmd2)
                        );

                    } else if (cmd.equals("treesmall")) {
                        cmd1 = loader.readLine();
                        cmd2 = loader.readLine();

                        addObject(
                            new TreeSmall(),
                            Integer.parseInt(cmd1),
                            Integer.parseInt(cmd2)
                        );

                    } else if (cmd.equals("treebig")) {
                        cmd1 = loader.readLine();
                        cmd2 = loader.readLine();

                        addObject(
                            new TreeBig(),
                            Integer.parseInt(cmd1),
                            Integer.parseInt(cmd2)
                        );

                    } else if (cmd.equals("turn")) {
                        cmd1 = loader.readLine();
                        cmd2 = loader.readLine();
                        cmd3 = loader.readLine();
                        cmd4 = loader.readLine();

                        addObject(
                            new Turn(
                                new Vector(
                                    Double.parseDouble(cmd3),
                                    Double.parseDouble(cmd4)
                            )), Integer.parseInt(cmd1),
                            Integer.parseInt(cmd2)
                        );

                    } else if (cmd.equals("powerup")) {
                        cmd1 = loader.readLine();
                        cmd2 = loader.readLine();
                        cmd3 = loader.readLine();

                        cmdType = Math.random() > 0.5 ? true : false;
                        if (cmd3.equals("mushroom"))
                            cmdType = true;
                        else if (cmd3.equals("flower"))
                            cmdType = false; 

                        addObject(
                            new PowerUp(cmdType),
                            Integer.parseInt(cmd1),
                            Integer.parseInt(cmd2)
                        );

                    } else if (cmd.equals("coin")) {
                        cmd1 = loader.readLine();
                        cmd2 = loader.readLine();
                        cmd3 = loader.readLine();
                        cmd4 = loader.readLine();

                        addObject(
                            new Coin(
                                new Vector(
                                    Double.parseDouble(cmd3),
                                    Double.parseDouble(cmd4)
                            )), Integer.parseInt(cmd1),
                            Integer.parseInt(cmd2)
                        );

                    } else if (cmd.equals("plant")) {
                        cmd1 = loader.readLine();
                        cmd2 = loader.readLine();

                        addObject(
                            new Plant(),
                            Integer.parseInt(cmd1),
                            Integer.parseInt(cmd2)
                        );

                    } else if (cmd.equals("goomba")) {
                        cmd1 = loader.readLine();
                        cmd2 = loader.readLine();
                        cmd3 = loader.readLine();

                        cmdType = Math.random() > 0.5 ? true : false;
                        if (cmd3.equals("left"))
                            cmdType = true;
                        else if (cmd3.equals("right"))
                            cmdType = false; 

                        addObject(
                            new Goomba(cmdType),
                            Integer.parseInt(cmd1),
                            Integer.parseInt(cmd2)
                        );

                    } else if (cmd.equals("koopa")) {
                        cmd1 = loader.readLine();
                        cmd2 = loader.readLine();
                        cmd3 = loader.readLine();

                        cmdType = Math.random() > 0.5 ? true : false;
                        if (cmd3.equals("left"))
                            cmdType = true;
                        else if (cmd3.equals("right"))
                            cmdType = false; 

                        addObject(
                            new Koopa(cmdType),
                            Integer.parseInt(cmd1),
                            Integer.parseInt(cmd2)
                        );

                    } else if (cmd.equals("bowser")) {
                        cmd1 = loader.readLine();
                        cmd2 = loader.readLine();
                        cmd3 = loader.readLine();
                        cmd4 = loader.readLine();
                        cmd5 = loader.readLine();

                        cmdBoss = new Bowser(Integer.parseInt(cmd5));

                        cmdType = Math.random() > 0.5 ? true : false;
                        if (cmd3.equals("left"))
                            cmdType = true;
                        else if (cmd3.equals("right"))
                            cmdType = false;
                        cmdBoss.direction = cmdType;

                        cmdType = Math.random() > 0.5 ? true : false;
                        if (cmd4.equals("left"))
                            cmdType = true;
                        else if (cmd4.equals("right"))
                            cmdType = false;
                        cmdBoss.face = cmdType;

                        addObject(
                            cmdBoss,
                            Integer.parseInt(cmd1),
                            Integer.parseInt(cmd2)
                        );

                    } else if (cmd.equals("player")) {
                        cmd1 = loader.readLine();
                        cmd2 = loader.readLine();

                        if (player != null)
                            removeObject(player);

                        player = new Mario();
                        addObject(
                            player,
                            Integer.parseInt(cmd1),
                            Integer.parseInt(cmd2)
                        );

                    } else if (cmd.equals("gravity")) {
                        cmd1 = loader.readLine();
                        cmd2 = loader.readLine();

                        gravity = new Vector(
                            Double.parseDouble(cmd1),
                            Double.parseDouble(cmd2)
                        );
                        up = new Vector(gravity.direction + Math.PI, gravity.magnitude);
                    }
                }
            
            loader.close();

        } catch (IOException e) {
            error(1);
            return false;
        }
        
        return true;
    }
}
