import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

public class Mario  extends People
{

    private int score = 0;
    
    private GreenfootImage jleft = new GreenfootImage("jumpLeft.png");
    private GreenfootImage jright = new GreenfootImage("jumpRight.png");
    private GreenfootImage sleft = new GreenfootImage("stillLeft.png");
    private GreenfootImage sright = new GreenfootImage("stillRight.png");
    
    private GreenfootImage[] wleft =
    {
        new GreenfootImage("walkLeft1.png"),
        new GreenfootImage("walkLeft2.png"),
        new GreenfootImage("walkLeft3.png"),
        new GreenfootImage("walkLeft2.png")
    };
    private GreenfootImage[] wright =
    {
        new GreenfootImage("walkRight1.png"),
        new GreenfootImage("walkRight2.png"),
        new GreenfootImage("walkRight3.png"),
        new GreenfootImage("walkRight2.png")
    };
    private int animate = 0;
    private int animationStep = 0;
    private boolean moving = false;

    public GreenfootImage lastImage;
    
    public Mario()
    {
        offset = 16;
        speed = 7;
        fall.max = 10;
        direction = false;
    }
    
    public void declarePlayer()
    {
        updateWorld();
        world.player = this;
    }
    
    public void act() 
    {
        updateWorld();
        physics();
        getJump();
        getWalk();
        platform();
        netForce.updatePosition();
        move();
        collect();
        kill();
    }
    
    private void getWalk()
    {  
        if (jumpNow) 
            if (direction) setImage(sleft); else setImage(sright);
        else
            if (direction) setImage(jleft); else setImage(jright); 
        
        int movement = 
            (Greenfoot.isKeyDown("left") ? 1 : 0) +
            (Greenfoot.isKeyDown("right") ? -1 : 0);
        switch (movement)
        {
            case 1: 
                netForce.add(left);
                direction = true;
                animation();
            break;
            case -1:
                netForce.add(right);
                direction = false;
                animation();
            break;
            default: 
                animate = 0;
                animationStep = 0;
        }
            
    }
    
    private void animation()
    {
        if (animationStep++ > 3)
        {
            animationStep = 0;
            animate = ++animate % 3;
        }
        if (jumpNow) setImage(direction ? wleft[animate] : wright[animate]);
    }
    
    private void getJump()
    {
        climbNow = Greenfoot.isKeyDown("up");
        if (climbNow && jump < maxJump)
        {
            jump++;
            jumpNow = false;
            netForce.add(jumpForce);
            fall.add(jumpForce);
            jumpSound = true;
        } else if (!jumpNow)
        {
            jump = maxJump;
            if (jumpSound)
            {
                jumpSound = false;
                Greenfoot.playSound("Jump.wav");
            }
        }
    }
    
    private void collect()
    {
        Coin money = (Coin)getOneIntersectingObject(Coin.class);
        if (money != null)
        {
            world.removeObject(money);
            Greenfoot.playSound("Coin.wav");
            score++;
            world.score.score++;
            world.score.update();
        }
        
        Block block = (Block)getOneIntersectingObject(Block.class);
        if (block != null)
        {
            block.bump(netForce);
            netForce.x *= -1;
            netForce.y *= -1;
            fall.x *= -1;
            fall.y *= -1;
        }
        
        Goal goal = (Goal)getOneIntersectingObject(Goal.class);
        if (goal != null)
        {
            world.goalReached(goal);
        }
        
        Turn turn = (Turn)getOneIntersectingObject(Turn.class);
        if (turn != null)
        {
            turn.Rotate();
            setRotation((int)Math.toDegrees(world.gravity.direction) - 90);
            jumpForce = new Vector(world.up.direction, world.up.magnitude * jumpPower);
            drop = new Vector(world.gravity.direction, world.gravity.magnitude);
            left = new Vector(world.gravity.direction + (Math.PI / 2), speed);
            right = new Vector(world.gravity.direction - (Math.PI / 2), speed);
        }
        
        PowerUp power = (PowerUp)getOneIntersectingObject(PowerUp.class);
        if (power != null && !world.invincible)
        {
            world.removeObject(power);
            lastImage = getImage();
            Greenfoot.playSound("PowerUp.wav");
            if (power.type)
            {
                setImage("yoshi.png");
                world.addObject(new SuperYoshi(this, world), getX(), getY());
            } else
            {
                setImage("fireAttack.png");
                Greenfoot.delay(70);
                setImage(lastImage);
                Greenfoot.playSound("Fireball.wav");
                int b = (int)(Math.random() * 20 + 10);
                for (int i = 0; i < b; i++) world.addObject(new YourFireball(), getX(), getY());
            }
        }
    }
    
    private void kill()
    {
        boolean dead = false;
        Koopa koopa = (Koopa)getOneIntersectingObject(Koopa.class);
        if (koopa != null)
        {
            Hurt target = (Hurt)getOneIntersectingObject(Hurt.class);
            if (target != null && koopa == target.target && !koopa.dead)
            {
                world.addObject(new Coin(Math.sqrt((fall.x * fall.x) + (fall.y * fall.y)) / 5), koopa.getX(), koopa.getY());
                world.addObject(new Coin(Math.sqrt((fall.x * fall.x) + (fall.y * fall.y)) / 4), koopa.getX(), koopa.getY());
                world.addObject(new Coin(Math.sqrt((fall.x * fall.x) + (fall.y * fall.y)) / 4), koopa.getX(), koopa.getY());
                world.addObject(new Coin(Math.sqrt((fall.x * fall.x) + (fall.y * fall.y)) / 3), koopa.getX(), koopa.getY());
                world.addObject(new Coin(Math.sqrt((fall.x * fall.x) + (fall.y * fall.y)) / 3), koopa.getX(), koopa.getY());
                world.removeObject(target);
                if (koopa.lives < 0) world.removeObject(koopa);
                fall.x *= -1;
                fall.y *= -1;
                Greenfoot.playSound("Hit.wav");
                koopa.dead = true;
            } else if (!koopa.dead) dead = true;
        }
        
        Goomba goomba = (Goomba)getOneIntersectingObject(Goomba.class);
        if (goomba != null)
        {
            Hurt target = (Hurt)getOneIntersectingObject(Hurt.class);
            if (target != null && goomba == target.target)
            {
                world.addObject(new Coin(Math.sqrt((fall.x * fall.x) + (fall.y * fall.y)) / 5), goomba.getX(), goomba.getY());
                world.addObject(new Coin(Math.sqrt((fall.x * fall.x) + (fall.y * fall.y)) / 4), goomba.getX(), goomba.getY());
                world.addObject(new Coin(Math.sqrt((fall.x * fall.x) + (fall.y * fall.y)) / 3), goomba.getX(), goomba.getY());
                world.removeObject(goomba);
                world.removeObject(target);
                fall.x *= -1;
                fall.y *= -1;
                Greenfoot.playSound("Hit.wav");
            } else dead = true;
        }
        
        if ((getX() < 3 || getX() > rt + 3) ||
            (getY() < 3 || getY() > dn + 3)
        ) dead = true;
        
        Bowser boss = (Bowser)getOneIntersectingObject(Bowser.class);
        if (boss != null) 
            if (Math.sqrt(
                (boss.getX() - getX()) * (boss.getX() - getX()) +
                (boss.getY() - getY()) * (boss.getY() - getY())
            ) < 40)
                dead = true;
         
        Fireball fire = (Fireball)getOneIntersectingObject(Fireball.class);
        if (fire != null && getOneIntersectingObject(YourFireball.class) == null)
            if (Math.sqrt(
                (fire.getX() - getX()) * (fire.getX() - getX()) +
                (fire.getY() - getY()) * (fire.getY() - getY())
            ) < 27)
                dead = true;
        
        if (getOneIntersectingObject(Plant.class) != null) dead = true;
        
        Weakpoint bossKill = (Weakpoint)getOneIntersectingObject(Weakpoint.class);
        if (bossKill != null) bossKill.defeat();
        
        if (dead) die();
    }
    
    private void die()
    {
        world.addObject(new Dead(), getX(), getY());
        world.removeObject(this);
        world.theme.stop();
        world.theme.close();
        world.theme.open("Dead.wav");
        world.theme.setLoops(1);
        world.theme.play();
        Greenfoot.delay(55);
        world.score.score = 0;
        world.score.update();
    }
}
