import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.util.List;

public class People extends Actor
{
    // Defines all variables per person
    public double mass = 1;
    protected double speed = 4;
    public Vector netForce = new Vector(0, 0);
    public Game world;
    
    protected int jump = 0;
    protected boolean jumpNow = false;
    protected boolean moveNow = true;
    protected int maxJump = 4;
    protected int offset = 0;
    protected double jumpPower = 10;
    protected Vector jumpForce;
    public Vector drop;
    protected Vector fall = new Vector(0, 0);
    protected Vector left;
    protected Vector right;
    protected boolean jumpSound = false;
    protected boolean jumpSoundNow = false;
    protected int climb = 0;
    protected boolean climbNow = true;
    protected int wallBump = 4;
    public boolean direction = Math.random() > 0.5 ? true : false;
    protected int rt;
    protected int dn;
    
    public void changeDirection()
    {
        direction = !direction;
    }
    
    protected void updateWorld()
    {
        if (world == null) 
        {
            world = (Game)getWorld();
            jumpForce = new Vector(world.up.direction, world.up.magnitude * jumpPower);
            drop = new Vector(world.gravity.direction, world.gravity.magnitude);
            left = new Vector(world.gravity.direction + (Math.PI / 2), speed);
            right = new Vector(world.gravity.direction - (Math.PI / 2), speed);
            rt = world.getWidth() - 10;
            dn = world.getHeight() - 10;
        }
    }
    
    protected void physics()
    {
        
        List floors = getIntersectingObjects(Floor.class);

        Floor ground;
        
        fall.add(drop);
        fall.updatePosition();
        netForce.x = fall.x;
        netForce.y = fall.y;
        
        for (int i = 0; i < floors.size(); i++)
        {
            ground = (Floor)floors.get(i);
            
            double len = Math.sqrt(
                (getY() - ground.getY()) * (getY() - ground.getY()) +
                (getX() - ground.getX()) * (getX() - ground.getX())
            );
            if (len < ground.radius + offset)
            {
                double pointX, pointY, dir;
                pointX = ground.getX() - getX();
                pointY = ground.getY() - getY();
                dir = Math.atan(pointY / pointX);
                dir += Math.PI / 2;
                if (pointX < 0) dir -= Math.PI;
                dir -= Math.PI / 2;
                Vector land = new Vector(dir + Math.PI, ground.radius + offset);
                drop = new Vector(dir, drop.magnitude);
                jumpForce = new Vector(dir + Math.PI, jumpPower);
                left = new Vector(dir + (Math.PI / 2), speed);
                right = new Vector(dir - (Math.PI / 2), speed);
                setRotation((int)Math.toDegrees(dir) - 90);
                netForce.x = netForce.y = fall.x = fall.y = 0;
                setLocation(ground.getX() + (int)land.x, ground.getY() + (int)land.y);
                jump = 0;
                jumpNow = true;
                jumpSound = false;
                //jumpSoundNow = true;
            }
        }
    }
    
    protected boolean platform()
    {
        List platforms = getIntersectingObjects(Brick.class);
        
        int _x, _y, __x, __y, u, d, l, r, _u, _d, _l, _r, vu, vd, vl, vr;
        Brick ground;
        
        int land = 0;
        boolean counted = false;
        
        _x = getX();
        _y = getY();
            
        for (int i = 0; i < platforms.size(); i++)
        {
            ground = (Brick)platforms.get(i);
            
            counted = false;
            
            u = ground.getY() - 18 - offset;
            d = ground.getY() + 18 + offset;
            l = ground.getX() - 16 - offset;
            r = ground.getX() + 16 + offset;
            __x = ground.getX();
            __y = ground.getY();
            vu = u - (int)(netForce.y / (mass + 0.0001));
            vd = d - (int)(netForce.y / (mass + 0.0001));
            vl = l - (int)(netForce.x / (mass + 0.0001));
            vr = r - (int)(netForce.x / (mass + 0.0001));
            _u = ground.getY() - 18;
            _d = ground.getY() + 18;
            _l = ground.getX() - 16;
            _r = ground.getX() + 16;
            
            if (_x > _l && _x < _r && _y > _u && _y < _d)
            {
                if (_x > _l && _x < __x)
                {
                    if (_y < __y)
                    {
                        _x = l;
                        _y = u;
                    } else if (_y > __y) {
                        _x = l;
                        _y = d;
                    }
                } else if (_x < _r && _x > __x) {
                    if (_y < __y)
                    {
                        _x = r;
                        _y = u;
                    } else if (_y > __y) {
                        _x = r;
                        _y = d;
                    }
                }
            } else {
                if (_y >= _u && _y <= _d)
                {
                    if (_x > vl && _x <= _l)
                    {
                        netForce.x = fall.x = ground.velocity.x;
                        _x = l;
                        _y += ground.velocity.y;
                        //newDir = 360; //270
                    } else if (_x < vr && _x >= _r) {
                        netForce.x = fall.x = ground.velocity.x;
                        _x = r;
                        _y += ground.velocity.y;
                        //newDir = 180; //90
                    }
                } 
                if (_x >= _l && _x <= _r) {
                    if (_y > vu && _y <= _u)
                    {
                        netForce.y = fall.y = ground.velocity.y;
                        _y = u;
                        _x += ground.velocity.x;
                        //newDir = 90; //0
                    } else if (_y < vd && _y >= _d) {
                        netForce.y = fall.y = ground.velocity.y;
                        _y = d;
                        _x += ground.velocity.x;
                        //newDir = 270; //180
                    }  
                }
            }
            
            land++;

            if (climb++ < 3)
            {
                jump = 0;
                jumpNow = true;
                jumpSound = false;
            } else climb = 3;
            if (!climbNow) climb = 0;
        }
        
        if (platforms.size() > 0)
        {
            setRotation((int)Math.toDegrees(world.gravity.direction) - 90);
            jumpForce = new Vector(world.up.direction, world.up.magnitude * jumpPower);
            drop = new Vector(world.gravity.direction, world.gravity.magnitude);
            left = new Vector(world.gravity.direction + (Math.PI / 2), speed);
            right = new Vector(world.gravity.direction - (Math.PI / 2), speed);
        }
        
        setLocation(_x, _y);
        
        return (land >= wallBump ? true : false);
    }
    
    protected void move()
    {
        setLocation(getX() + (int)(netForce.x / (mass + 0.0001)), getY() + (int)(netForce.y / (mass + 0.0001)));
        //if (jumpSound && jumpSoundNow)
        //{
            //jumpSoundNow = false;
       //     Greenfoot.playSound("Jump.wav");
       // }
    }
    
    protected void atEgde()
    {
        /*if (getX() < 10 || getX() > world.getWidth() - 10) 
        {
            netForce.x *= -1;
            fall.x *= -1;
        }
        if (getY() < 10 || getY() > world.getHeight() - 10)
        {
            netForce.y *= -1;
            fall.y *= -1;
        }*/
        int _y, _x;
        _x = getX();
        _y = getY();
        if (_x < 10) setLocation(rt, _y);
        if (_x > rt) setLocation(10, _y);
        if (_y < 10) setLocation(_x, dn);
        if (_y > dn) setLocation(_x, 10);
    }
}
